// ----------------------------------------------------
// Includes
#include "psp.h"

// ----------------------------------------------------
// Fucntions
extern const gchar *(*gimp_plug_in_directory_)(void);

// ----------------------------------------------------
// Variables
char *Config_Keys[] = {
	"Dest_Format",
	"Dest_Palette_Format",
	"Nbr_MipMaps",
	"Use_Alpha_Color",
	"Alpha_Color",
	"Swizzle_Texture",
	"Use_Swizzle_Texture"
};
char Config_FileName[512];

// ----------------------------------------------------
// Check the validity of a key
int Get_Config_Key(char *KeyName) {
	int i;
	for(i = 0; i < 7; i++) {
		if(!stricmp(KeyName, Config_Keys[i])) return(i);
	}
	return(-1);
}

// ----------------------------------------------------
// Fill the default values structure from the config file
void Load_Config(LPPluginValues Values) {
	FILE *Config_File;
	char Config_Key[64];
	int Config_Value;
	int i;
	int Config_Idx;
#ifdef G_OS_WIN32
	sprintf(Config_FileName, "%s\\psptex.config", gimp_plug_in_directory_());
#else
	sprintf(Config_FileName, "%s\\psptex.config", gimp_plug_in_directory());
#endif
	Config_File = fopen(Config_FileName, "r");
	if(Config_File) {
		fscanf(Config_File, "%s", &Config_Key);
		if(!stricmp(Config_Key, "PSPTEX")) {
			for(i = 0; i < sizeof(Config_Keys) / 4; i++) {
				fscanf(Config_File, "%s %d", &Config_Key, &Config_Value);
				Config_Idx = Get_Config_Key(Config_Key);
				if(Config_Idx == -1) break;
				switch(Config_Idx) {
					case 0:
						Values->Dest_Format = Config_Value;
						break;
					case 1:
						Values->Dest_Palette_Format = Config_Value;
						break;
					case 2:
						Values->Nbr_MipMaps = Config_Value;
						break;
					case 3:
						Values->Use_Alpha_Color = Config_Value;
						break;
					case 4:
						Values->Alpha_Color = Config_Value;
						break;
					case 5:
						Values->Swizzle_Texture = Config_Value;
						break;
					case 6:
						Values->Use_Swizzle_Texture = Config_Value;
						break;
				}
			}
			switch(Values->Dest_Format) {
				case GU_PSM_DXT1:
				case GU_PSM_DXT3:
				case GU_PSM_DXT5:
					Values->Use_Swizzle_Texture = FALSE;
					break;
			}
		}
		fclose(Config_File);
	}				
}

// ----------------------------------------------------
// Save the config file
void Save_Config(LPPluginValues Values) {
	FILE *Config_File;
#ifdef G_OS_WIN32
	sprintf(Config_FileName, "%s\\psptex.config", gimp_plug_in_directory_());
#else
	sprintf(Config_FileName, "%s\\psptex.config", gimp_plug_in_directory());
#endif
	Config_File = fopen(Config_FileName, "w");
	if(Config_File) {
		fprintf(Config_File, "PSPTEX\n");
		fprintf(Config_File, "%s %d\n", Config_Keys[0], Values->Dest_Format);
		fprintf(Config_File, "%s %d\n", Config_Keys[1], Values->Dest_Palette_Format);
		fprintf(Config_File, "%s %d\n", Config_Keys[2], Values->Nbr_MipMaps);
		fprintf(Config_File, "%s %d\n", Config_Keys[3], Values->Use_Alpha_Color);
		fprintf(Config_File, "%s %d\n", Config_Keys[4], Values->Alpha_Color);
		fprintf(Config_File, "%s %d\n", Config_Keys[5], Values->Swizzle_Texture);
		fprintf(Config_File, "%s %d\n", Config_Keys[6], Values->Use_Swizzle_Texture);
		fclose(Config_File);
	}
}
